<?php
if (!defined('ABSPATH')) {
	exit();
}

if (!class_exists('App_Expert_Users_Controller')) {

	class App_Expert_Users_Controller extends  WP_REST_Users_Controller
	{

		const USER_IMAGE_SMALL = 200;
		const USER_IMAGE_MEDIUM = 350;
		const USER_IMAGE_LARGE = 650;

		public function __construct()
		{

			parent::__construct();
			$this->rest_base = '/users';
			$this->namespace = APP_EXPERT_API_NAMESPACE;
		}

		public function register_routes()
		{

			register_rest_route(
				$this->namespace,
				'/' . $this->rest_base . '/me',
				array(
					array(
						'methods'  => WP_REST_Server::READABLE,
						'callback' => array($this, 'get_current_item'),
					),
					array(
						'methods'             => WP_REST_Server::EDITABLE,
						'callback'            => array($this, 'update_current_item'),
						'args'                => $this->get_endpoint_args_for_item_schema(WP_REST_Server::EDITABLE),
					),
					'schema' => array($this, 'get_public_item_schema'),
				)
			);
		}

		public function get_current_item($request)
		{
			$user_id = App_Expert_Request::get_user_id($request);
			$user     = $this->get_user((int) $user_id);

			if (is_wp_error($user))
				return $user;


			$response = $this->prepare_item_for_response($user, $request);
			$response = rest_ensure_response($response);

			return App_Expert_Response::get_api_single_success_response(
				$response,
				'app_expert_retrieved_user',
				__('Profile data retrieved successfully.', 'app-expert')
			);
		}

		public function update_current_item($request)
		{
			$user_id = App_Expert_Request::get_user_id($request);
			if (isset($user_id) && !empty($user_id))
				$request['id'] = $user_id;


			$request = App_Expert_Request::remove_user_meta_data_update($request);
			$validation_response = $this->validate_user_names_data($request);
			if (is_wp_error($validation_response))
				return $validation_response;


			$response = parent::update_item($request);

			if (is_wp_error($response))
				return $response;

			return App_Expert_Response::get_api_single_success_response(
				$response,
				'app_expert_updated_user',
				__('Profile data updated successfully.', 'app-expert')
			);
		}



		public function prepare_item_for_response($user, $request)
		{
			return App_Expert_Response::prepare_user_data_for_response($user);
		}

		public function get_item_schema()
		{

			return array(
				'$schema'    => 'http://json-schema.org/draft-04/schema#',
				'title'      => 'user',
				'type'       => 'object',
				'properties' => array(
					'first_name'         => array(
						'description' => __('First name for the user.', 'app-expert'),
						'type'        => 'string',
						'context'     => array('edit'),
						'arg_options' => array(
							'sanitize_callback' => 'sanitize_text_field',
						),
					),
					'last_name'          => array(
						'description' => __('Last name for the user.', 'app-expert'),
						'type'        => 'string',
						'context'     => array('edit'),
						'arg_options' => array(
							'sanitize_callback' => 'sanitize_text_field',
						),
					),
					'email'              => array(
						'description' => __('The email address for the user.', 'app-expert'),
						'type'        => 'string',
						'format'      => 'email',
						'context'     => array('edit'),
					),

				),
			);
		}


			/**
		 * @param WP_REST_Request $request
		 *
		 * @return WP_Error|bool
		 */
		private function validate_user_names_data($request)
		{

			$first_name = $request->get_param("first_name");
			$last_name = $request->get_param("last_name");
			if (!$first_name) {
				return $this->get_validation_error("missing_first_name", "first_name", __("Invalid parameter(s): first_name", "app-expert"), __("The first name field is empty", "app-expert"));
			}
			if (!$last_name) {
				return $this->get_validation_error("missing_last_name", "last_name", __("Invalid parameter(s): last_name", "app-expert"), __("The last name field is empty", "app-expert"));
			}
			if (strlen($first_name) > 50) {
				return $this->get_validation_error("invalid_first_name_length", "first_name", __("Invalid parameter(s): first_name", "app-expert"), __("The first name field must not exceed 50 chatacters", "app-expert"));
			}
			if (strlen($last_name) > 50) {
				return $this->get_validation_error("invalid_last_name_length", "last_name", __("Invalid parameter(s): last_name", "app-expert"), __("The last name field must not exceed 50 chatacters", "app-expert"));
			}

			return true;
		}

		private function get_validation_error($key, $field, $message, $error_message)
		{

			return new WP_Error(
				$key,
				$message,
				array(
					'status' => 400,
					'params' => array(
						$field => $error_message
					)
				)
			);
		}

	}
}


$app_expert_users_controller = new App_Expert_Users_Controller();
$app_expert_users_controller->register_routes();
